/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2017-2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "misc.h"
#include "inttostr.h"
#include "utf8/utf8.h"


namespace pt
{


void SetOverflow(bool * was_overflow, bool val)
{
	if( was_overflow )
		*was_overflow = val;
}


void esc_to_json(char val, Stream & out)
{
	if( (unsigned char)val < 32 )
	{
		char buf[10];
		size_t len;
		Toa((unsigned char)val, buf, sizeof(buf)/sizeof(char), 16, &len);

		out << "\\u";

		if( len < 4 )
		{
			for(size_t i=0 ; i < (4-len) ; ++i)
			{
				out << '0';
			}
		}

		out << buf;
	}
	else
	{
	// CHECKME
	// \r \n \t are <32 and will be serialized os \u.... above

		switch( val )
		{
		case 0:		out << '\\';	out << '0';		break;	// may to skip this character is better?
		case '\r':	out << '\\';	out << 'r';		break;
		case '\n':	out << '\\';	out << 'n';		break;
		case '\t':	out << '\\';	out << 't';		break;
		case 0x08:	out << '\\';	out << 'b';		break;
		case 0x0c:	out << '\\';	out << 'f';		break;
		case '\\':	out << '\\';	out << '\\';		break;
		case '"':	out << '\\';	out << '\"';		break;
		default:
			out << val;
		}
	}
}


void esc_to_json(wchar_t val, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);

	size_t len = int_to_utf8(static_cast<int>(val), utf8_buf, utf8_buf_len);

	for(size_t a = 0 ; a < len ; ++a)
	{
		esc_to_json(utf8_buf[a], out);
	}
}


void esc_to_json(const char * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_json(c[i], out);
	}
}


void esc_to_json(const char * c, std::size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_json(c[i], out);
	}
}


void esc_to_json(const wchar_t * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_json(c[i], out);
	}
}


void esc_to_json(const wchar_t * c, size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_json(c[i], out);
	}
}


void esc_to_json(const std::string & in, Stream & out)
{
	esc_to_json(in.c_str(), in.size(), out);
}


void esc_to_json(const std::wstring & in, Stream & out)
{
	esc_to_json(in.c_str(), in.size(), out);
}






void esc_to_xml(char val, Stream & out)
{
	switch(val)
	{
	case '<':
		out << "&lt;";
		break;

	case '>':
		out << "&gt;";
		break;

	case '&':
		out << "&amp;";
		break;

	case '"':
		out << "&quot;";
		break;

	default:
		out << val;
		break;

	// what about zero (null) character?
	}
}

void esc_to_xml(wchar_t val, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);

	size_t len = int_to_utf8(static_cast<int>(val), utf8_buf, utf8_buf_len);

	for(size_t a = 0 ; a < len ; ++a)
	{
		esc_to_xml(utf8_buf[a], out);
	}
}


void esc_to_xml(const char * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_xml(c[i], out);
	}
}


void esc_to_xml(const char * c, std::size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_xml(c[i], out);
	}
}


void esc_to_xml(const wchar_t * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_xml(c[i], out);
	}
}


void esc_to_xml(const wchar_t * c, size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_xml(c[i], out);
	}
}


void esc_to_xml(const std::string & in, Stream & out)
{
	esc_to_xml(in.c_str(), in.size(), out);
}


void esc_to_xml(const std::wstring & in, Stream & out)
{
	esc_to_xml(in.c_str(), in.size(), out);
}





void esc_to_csv(char c, pt::Stream & out)
{
	switch(c)
	{
	case '"':
		out << "\"\"";
		break;

	default:
		out << c;
		break;

	// what about zero (null) character?
	}
}


void esc_to_csv(wchar_t val, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);

	size_t len = int_to_utf8(static_cast<int>(val), utf8_buf, utf8_buf_len);

	for(size_t a = 0 ; a < len ; ++a)
	{
		esc_to_csv(utf8_buf[a], out);
	}
}



void esc_to_csv(const char * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_csv(c[i], out);
	}
}


void esc_to_csv(const char * c, std::size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_csv(c[i], out);
	}
}


void esc_to_csv(const wchar_t * c, pt::Stream & out)
{
	for(size_t i = 0 ; c[i] != 0 ; ++i)
	{
		esc_to_csv(c[i], out);
	}
}


void esc_to_csv(const wchar_t * c, size_t len, pt::Stream & out)
{
	for(size_t i = 0 ; i < len ; ++i)
	{
		esc_to_csv(c[i], out);
	}
}


void esc_to_csv(const std::string & in, Stream & out)
{
	esc_to_csv(in.c_str(), in.size(), out);
}


void esc_to_csv(const std::wstring & in, Stream & out)
{
	esc_to_csv(in.c_str(), in.size(), out);
}




}

