/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2017-2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_picotools_convert_misc
#define headerfile_picotools_convert_misc

#include <limits>
#include "text.h"
#include "textstream/stream.h"
#include "textstream/types.h"
#include "utf8/utf8_stream.h"


namespace pt
{

void SetOverflow(bool * was_overflow, bool val);

void esc_to_json(char val, Stream & out);
void esc_to_json(wchar_t val, Stream & out);
void esc_to_json(const char * c, pt::Stream & out);
void esc_to_json(const char * c, std::size_t len, Stream & out);
void esc_to_json(const wchar_t * c, Stream & out);
void esc_to_json(const wchar_t * c, size_t len, pt::Stream & out);
void esc_to_json(const std::string & in, Stream & out);
void esc_to_json(const std::wstring & in, Stream & out);

void esc_to_xml(char c, pt::Stream & out);
void esc_to_xml(wchar_t c, pt::Stream & out);
void esc_to_xml(const char * c, pt::Stream & out);
void esc_to_xml(const char * c, std::size_t len, pt::Stream & out);
void esc_to_xml(const wchar_t * c, pt::Stream & out);
void esc_to_xml(const wchar_t * c, size_t len, pt::Stream & out);
void esc_to_xml(const std::string & in, Stream & out);
void esc_to_xml(const std::wstring & in, Stream & out);

void esc_to_csv(char c, pt::Stream & out);
void esc_to_csv(wchar_t val, Stream & out);
void esc_to_csv(const char * c, std::size_t len, Stream & out);
void esc_to_csv(const char * c, pt::Stream & out);
void esc_to_csv(const char * c, std::size_t len, pt::Stream & out);
void esc_to_csv(const wchar_t * c, pt::Stream & out);
void esc_to_csv(const wchar_t * c, size_t len, pt::Stream & out);
void esc_to_csv(const std::string & in, Stream & out);



template<typename StreamType>
void esc_to_json(const StreamType & in, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);
	typename StreamType::const_iterator i = in.begin();
	typename StreamType::const_iterator end = in.end();
	int res;
	bool correct;

	for( ; i != end ; ++i)
	{
		if( in.is_wchar_stream() && out.is_char_stream() )
		{
			std::size_t len = int_to_utf8(static_cast<int>(*i), utf8_buf, utf8_buf_len);
			esc_to_json(utf8_buf, len, out);
		}
		else
		if( in.is_char_stream() && out.is_wchar_stream() )
		{
			utf8_to_int(i, end, res, correct);

			if( correct )
				esc_to_json(static_cast<wchar_t>(res), out); // IMPROVEME no surrogate pair used here (if sizeof(wchar_t) == 2)

			// put replacement char if not correct?
		}
		else
		{
			esc_to_json(static_cast<wchar_t>(*i), out);
		}
	}
}


template<typename StreamType>
void esc_to_xml(const StreamType & in, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);
	typename StreamType::const_iterator i = in.begin();
	typename StreamType::const_iterator end = in.end();
	int res;
	bool correct;

	for( ; i != end ; ++i)
	{
		if( in.is_wchar_stream() && out.is_char_stream() )
		{
			std::size_t len = int_to_utf8(static_cast<int>(*i), utf8_buf, utf8_buf_len);
			esc_to_xml(utf8_buf, len, out);
		}
		else
		if( in.is_char_stream() && out.is_wchar_stream() )
		{
			utf8_to_int(i, end, res, correct);

			if( correct )
				esc_to_xml(static_cast<wchar_t>(res), out); // IMPROVEME no surrogate pair used here (if sizeof(wchar_t) == 2)

			// put replacement char if not correct?
		}
		else
		{
			esc_to_xml(static_cast<wchar_t>(*i), out);
		}
	}
}


template<typename StreamType>
void esc_to_csv(const StreamType & in, Stream & out)
{
	char utf8_buf[10];
	std::size_t utf8_buf_len = sizeof(utf8_buf) / sizeof(char);
	typename StreamType::const_iterator i = in.begin();
	typename StreamType::const_iterator end = in.end();
	int res;
	bool correct;

	for( ; i != end ; ++i)
	{
		if( in.is_wchar_stream() && out.is_char_stream() )
		{
			std::size_t len = int_to_utf8(static_cast<int>(*i), utf8_buf, utf8_buf_len);
			esc_to_csv(utf8_buf, len, out);
		}
		else
		if( in.is_char_stream() && out.is_wchar_stream() )
		{
			utf8_to_int(i, end, res, correct);

			if( correct )
				esc_to_csv(static_cast<wchar_t>(res), out); // IMPROVEME no surrogate pair used here (if sizeof(wchar_t) == 2)

			// put replacement char if not correct?
		}
		else
		{
			esc_to_csv(static_cast<wchar_t>(*i), out);
		}
	}
}

}


#endif

